/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Per-branch ACL implementation

*/

#ifndef BRANCH_ACL_H
#define BRANCH_ACL_H 1

#include "dir-acl.h"
#include "user-acl.h"

#include "stringbuf.h"

struct BRANCH_ACL;

struct BRANCH_ACE {
    stringbuf *fullpath;
    stringbuf *filename;
    struct BRANCH_ACL *acl;
    USER_ACL *user_acls;
    struct BRANCH_ACE *next;
};

typedef struct BRANCH_ACE BRANCH_ACE;

struct BRANCH_ACL {
    struct DIR_ACL *dir_acl;
    char *branch;
    struct BRANCH_ACE *first_ace;
    struct BRANCH_ACL *next;
};

typedef struct BRANCH_ACL BRANCH_ACL;

enum BRANCH_PERM {
    branch_perm_none = 0x00,
    branch_perm_checkout = 0x01,
    branch_perm_checkin = 0x02,
    branch_perm_invalid = 0x80,
    branch_perm_all = branch_perm_checkout | branch_perm_checkin,
    default_branch_perm = branch_perm_checkout | branch_perm_checkin
};

typedef int branch_permission;

BRANCH_ACL *create_branch_acl (DIR_ACL *dir_acl, char *branch);
BRANCH_ACL *find_branch_acl (DIR_ACL *dir_acl, char *branch);
BRANCH_ACL *get_branch_acl (DIR_ACL *dir_acl, char *branch);

BRANCH_ACE *get_file_on_branch_ace (BRANCH_ACL *branch_acl, char *file);

int set_branch_ace (BRANCH_ACE *branch_ace, char *username,
		    branch_permission perm);

int branch_permission_granted (BRANCH_ACE *branch_acl, char *username, 
			       branch_permission perm);


int parse_branch_permissions (char *perm_str);
char *branch_permissions_string (int perm);

int internal_read_branch_acl_file (char *filename, char *on_branch, struct BRANCH_ACE *bacl);

#endif
