/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Per-directory ACL implementation

*/

#ifndef DIR_ACL_H
#define DIR_ACL_H 1

#include "stringbuf.h"

struct BRANCH_ACL;

struct DEFAULT_FILE_ACL {
    char *branch;
    struct USER_ACL *user_acl;
    struct DEFAULT_FILE_ACL *next;
};

struct DIR_ACL {
    stringbuf *dirname;
    struct USER_ACL *user_acls;
    struct DEFAULT_FILE_ACL *default_file_acl;
    struct DIR_ACL **top_dirs;
    struct BRANCH_ACL *branch_acls;
    int top_dirs_count;
    int top_dirs_allocated;
    int branch_acls_loaded;
};

enum DIR_PERM {
    dir_perm_none = 0x00,
    dir_perm_access = 0x01,
    dir_perm_modify = 0x02,
    dir_perm_invalid = 0x80,
    dir_perm_all = dir_perm_access | dir_perm_modify
};

typedef struct DIR_ACL DIR_ACL;

typedef int dir_permission;

void set_toplevel_directory(char *toplevel_dir);

void set_default_dir_perm (enum DIR_PERM new_perm);
enum DIR_PERM get_default_dir_perm (void);

DIR_ACL *get_dir_acl(char *dir, char *branch);

int load_branch_acls(struct DIR_ACL *dir_acl);
int store_branch_acls (struct DIR_ACL *dir_acl);


int parse_dir_permissions(char *perm_string);
char *dir_permissions_string(int perm);

int set_dir_ace (struct DIR_ACL *dir_acl, char *username, int perm);

int dir_permission_granted (struct DIR_ACL *dir_acl, char *username, int perm);

int store_dir_acl (struct DIR_ACL *dir_acl);

int set_default_file_acl (DIR_ACL *dir_acl, char *branch, char *username, int perm);

struct USER_ACL *get_default_file_user_acl(DIR_ACL *dir_acl, char *branch);

#endif /* DIR_ACL_H */
