#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "stringbuf.h"

#include <assert.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif

#ifdef HAVE_STRING_H
#include <string.h>
#endif

struct stringbuf *
new_stringbuf (char *str)
{
    struct stringbuf *sb;
    size_t allocated;

    if (str == NULL)
	return NULL;

    sb = (struct stringbuf *) malloc(sizeof(struct stringbuf));
    if (sb == NULL)
	return NULL;

    sb->len = strlen(str);

    if (sb->len == 0)
	allocated = 8;
    else
	allocated = sb->len * 2;

    sb->buf = (char *) malloc(allocated);
    if (sb->buf == NULL) {
	free(sb);
	return NULL;
    }

    strcpy(sb->buf, str);

    sb->allocated = allocated;

    return sb;
}


int
free_stringbuf (struct stringbuf *buf)
{
    if (buf == NULL)
	return 1;

    free(buf->buf);
    buf->buf = NULL;

    buf->allocated = 0;
    buf->len = 0;
    
    return 1;
}


struct stringbuf *
dup_stringbuf (struct stringbuf *buf)
{
    struct stringbuf *newbuf;

    if (buf == NULL)
	return NULL;

    newbuf = (struct stringbuf *) malloc(sizeof(struct stringbuf));
    if (newbuf == NULL)
	return NULL;

    newbuf->len = buf->len;
    newbuf->allocated = buf->allocated;
    newbuf->buf = (char *) malloc(buf->allocated);
    if (newbuf->buf == NULL) {
	free(newbuf);
	return NULL;
    }
    strcpy(newbuf->buf, buf->buf);

    return newbuf;
}

struct stringbuf *
catc_stringbuf (struct stringbuf *buf, char c)
{
    if (buf->len + 1 >= buf->allocated) {
	size_t needs_to_allocate = buf->allocated * 2;
	char *tmp = (char *) realloc(buf->buf, needs_to_allocate);
	if (tmp == NULL)
	    return NULL;
	buf->buf = tmp;
	buf->allocated = needs_to_allocate;
    }
    buf->buf[buf->len++] = c;
    buf->buf[buf->len] = '\0';

    return buf;
}

struct stringbuf *
cat_stringbuf (struct stringbuf *buf, char *str)
{
    size_t len;

    if (buf == NULL)
	return NULL;

    if (str == NULL)
	return buf;

    len = strlen(str);
    if (len == 0)
	return buf;

    if (buf->len + len >= buf->allocated) {
	size_t needs_to_allocate = buf->allocated + len * 2 + 1;
	char *tmp = (char *) realloc(buf->buf, needs_to_allocate);
	if (tmp == NULL)
	    return NULL;
	buf->buf = tmp;
	buf->allocated = needs_to_allocate;
    }

    strcpy(buf->buf + buf->len, str);
    buf->len += len;

    return buf;
}


struct stringbuf *
shrink_stringbuf (struct stringbuf *buf, size_t newlen)
{

    if (newlen > buf->len)
	return NULL;
    
    buf->len = newlen;
    buf->buf[buf->len] = '\0';

    return buf;
}
