/* 
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   Copyright (c) Alexey Mahotkin <alexm@hsys.msk.ru> 2001

   Routines for authentication against :pserver: 

*/

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "error.h"
#include "login.h"
#include "root.h"
#include "network-client.h"

#include "client_pserver_auth.h"


static void
ncli_sendstr(NCLI ncli, char *str)
{
    int len = strlen(str);
    ncli->write(ncli, str, len);
}

static void
ncli_send_lf(NCLI ncli)
{
    ncli->write(ncli, "\x0a", 1);
}

int
authenticate_against_pserver (NCLI ncli, int verify_only)
{
    char *begin_request = verify_only ?
	"BEGIN VERIFICATION REQUEST\012" : "BEGIN AUTH REQUEST\012";

    char *end_request = verify_only ?
	"END VERIFICATION REQUEST\012" : "END AUTH REQUEST\012";

    char *read_buf = NULL;
    int passed = 0;

    /* Get the password, probably from ~/.cvspass. */
    char *password = get_cvs_password ();

    /* Send the empty string by default.  This is so anonymous CVS
       access doesn't require client to have done "cvs login". */
    if (password == NULL) 
    {
	password = scramble ("");
	/* Output a special error message if authentication was attempted
	   with no password -- the user should be made aware that they may
	   have missed a step. */
	error (0, 0,
	       "used empty password; try \"cvs login\" with a real password");
    }

    /* Announce that we're starting the authorization protocol. */
    ncli_sendstr(ncli, begin_request);
    /* Send the data the server needs. */
    ncli_sendstr(ncli, current_parsed_root->directory);
    ncli_send_lf(ncli);
    ncli_sendstr(ncli, current_parsed_root->username);
    ncli_send_lf(ncli);
    ncli_sendstr(ncli, password);
    ncli_send_lf(ncli);
    /* Announce that we're ending the authorization protocol. */
    ncli_sendstr(ncli, end_request);

    /* Paranoia. */
    memset (password, 0, strlen (password));

    /* Loop, getting responses from the server.  */
    while (1)
    {
	read_buf = ncli_read_line (ncli);
	if (read_buf == NULL) {
	    error(1, 0, "Cannot read line from %s:%d %s",
		  current_parsed_root->hostname, current_parsed_root->port,
		  strerror(errno));
	}

	if (strcmp (read_buf, "I HATE YOU") == 0)
	{
	    /* Authorization not granted. */
	    break;
	}
	else if (strncmp (read_buf, "E ", 2) == 0)
	{
	    fprintf (stderr, "%s\n", read_buf + 2);

	    /* Continue with the authentication protocol.  */
	}
	else if (strncmp (read_buf, "error ", 6) == 0)
	{
	    char *p;

	    /* First skip the code.  */
	    p = read_buf + 6;
	    while (*p != ' ' && *p != '\0')
		++p;

	    /* Skip the space that follows the code.  */
	    if (*p == ' ')
		++p;

	    /* Now output the text.  */
	    fprintf (stderr, "%s\n", p);
	    break;
	}
	else if (strcmp (read_buf, "I LOVE YOU") == 0)
	{
	    passed = 1;
	    break;
	}
	else
	{
	    /* Unrecognized response from server. */
	    error (0, 0,
		   "unrecognized auth response from %s:%d %s", 
		   current_parsed_root->hostname, current_parsed_root->port,
		   read_buf);
	    break;
	}
    }

    free (read_buf);
    return passed;
}
