/* 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.  
   
   Copyright (C) Alexey Mahotkin <alexm@hsys.msk.ru> 1999-2000

   Reads CVS user name, password and repository, using :nserver:
   protocol.

*/

#define _BSD_SOURCE 1

#include "config.h"

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <unistd.h>

#include "cvs-server.h"
#include "env.h"

char *usage = 
"Usage: %s /repos1 ... /reposN -- hostname subprogram\n\
This program is designed to be run from TCP-server like inetd\n\n";

#define BUFFER_SIZE 512

static void
invalid_username_repository (void)
{
    puts("500 Invalid username@/repository");
    fflush(stdout);
    exit(EXIT_SUCCESS);
}

static void
out_of_memory (void)
{
    puts("500 Virtual memory exhausted");
    fflush(stdout);
    exit(EXIT_FAILURE);
}

int
main (int argc, char *argv[])
{
    int anonymous;
    char *buffer;
    int errorlevel;
    char **hostname = NULL;
    char **subprog;
    char *repository;
    char *username;
    char *password;
    int pi[2];
    pid_t pid;
    int wstat;
    
    if (argc <= 3)
    {
	printf (usage, argv[0]);
	exit (EXIT_FAILURE);
    }

    hostname = argv + 1;
    while (hostname) {
	if (strcmp(*hostname, "--") == 0)
	    break;
	hostname++;
    }

    if (!hostname)
    {
	printf (usage, argv[0]);
	exit (EXIT_FAILURE);
    }
    
    hostname++; 

    subprog = hostname + 1;
    if (!*subprog)
    {
	printf (usage, argv[0]);
	exit (EXIT_FAILURE);
    }

    buffer = malloc(BUFFER_SIZE);
    if (!buffer)
	out_of_memory ();

    printf("320 This is CVS nserver 1.10.7 at %s\n", *hostname);
    puts("310 Please enter username@/repository");
    fflush(stdout);
    
    get_buffer(buffer, BUFFER_SIZE);
    
    repository = strchr(buffer, '@');
    if (!repository)
	invalid_username_repository ();
    
    *repository++ = '\0';
    repository = strdup(repository);
    if (!repository)
	out_of_memory ();

    username = strdup(buffer);
    if (!username)
	out_of_memory ();

    anonymous = strcmp(username, "anonymous") == 0;

    if (anonymous)
	puts("310 Please enter your e-mail address as a password");
    else
	printf("310 Please enter password for %s at %s\n", username, repository);
    fflush(stdout);

    get_buffer(buffer, BUFFER_SIZE);

    password = strdup(buffer);
    if (!password)
	out_of_memory ();

    strip_trailing_slash (repository);

    if (!repository_ok (repository, argc, argv))
	invalid_username_repository ();

    setenvvar("REPOSITORY", repository);
    setenvvar("CVSUSER", username);
    setenvvar("CVSSERVER", "nserver");
    if (anonymous)
	setenvvar("ANONYMOUS", password);

    if (!create_pipe (pi))
    {
	fprintf (stderr, "500 %s: cannot create a pipe: %s\n", argv[0], strerror(errno));
	exit (1);
    }

    pid = fork ();

    if (pid == -1)
    {
	fprintf (stderr, "500 %s: cannot fork(): %s\n", argv[0], strerror(errno));
	exit (1);
    }

    if (pid == 0)
    {
	close(pi[1]);
	execvp(*subprog, subprog);
	fprintf (stderr, "500 %s: cannot execvp(%s): %s\n", 
		 argv[0], *subprog, strerror(errno));
	_exit(1);
    }

    close(pi[0]);
    write(pi[1], username, strlen(username) + 1);
    write(pi[1], password, strlen(password) + 1);
    write(pi[1], repository, strlen(repository) + 1);
    close(pi[1]);
    memset(password, 0, strlen(password));

    if (waitpid(pid, &wstat, 0) == -1)
    {
	fprintf (stderr, "500 %s: waitpid() returned -1: %s\n", argv[0], strerror(errno));
	exit (1);
    }

    if (WIFSIGNALED(wstat)) {
	fprintf (stderr, "500 %s: %s crashed\n", argv[0], *subprog);
	exit (1);
    }

    errorlevel = WEXITSTATUS(wstat);

    if (errorlevel == 2)
	printf("500 cvs-pserver: Internal error: %s misuse\n", *subprog);

    if ((errorlevel >= 111) && (errorlevel <= 120))
	printf("500 cvs-pserver: Temporary %s error %d\n", *subprog, errorlevel);

    if (errorlevel != 0)
	invalid_username_repository ();

    exit (0);
}
